@@ -3,7 +3,7 @@ import asyncio
 from itertools import chain
 from os import PathLike
 from pathlib import Path
-from typing import ClassVar, List, Sequence, Type, Union
+from typing import ClassVar, Iterable, List, Sequence, Type, Union
 
 from diot import Diot
 from rich import box
@@ -15,7 +15,7 @@ from simpleconf import Config
 from slugify import slugify  # type: ignore
 
 from .defaults import CONFIG, CONFIG_FILES, CONSOLE_WIDTH
-from .exceptions import ProcDependencyError
+from .exceptions import ProcDependencyError, PipenSetDataError
 from .pluginmgr import plugin
 from .proc import Proc
 from .progressbar import PipelinePBar
@@ -41,6 +41,7 @@ class Pipen:
         starts: The start processes
         config: The configurations
         workdir: The workdir for the pipeline
+        profile: The profile of the configurations to run the pipeline
         _kwargs: The extra configrations passed to overwrite the default ones
 
         PIPELINE_COUNT: How many pipelines are loaded
@@ -68,8 +69,11 @@ class Pipen:
         self.pbar: PipelinePBar = None
         self.name = name or f"pipen-{self.__class__.PIPELINE_COUNT}"
         self.desc = desc
-        self.outdir = Path(outdir or f"./{slugify(self.name)}_results")
+        self.outdir = Path(
+            outdir or f"./{slugify(self.name)}_results"
+        ).resolve()
         self.workdir: Path = None
+        self.profile: str = "default"
 
         self.starts: List[Proc] = []
         self.config = Diot(copy_dict(CONFIG, 3))
@@ -77,7 +81,7 @@ class Pipen:
         # the profile yet
         self._kwargs = kwargs
 
-        if not self.__class__.SETUP:
+        if not self.__class__.SETUP:  # pragma: no cover
             # Load plugins from entrypotins at runtime to avoid
             # cyclic imports
             plugin.load_entrypoints()
@@ -89,8 +93,8 @@ class Pipen:
         # make sure main plugin is enabled
         plugin.get_plugin("main").enable()
 
-        if not self.__class__.SETUP:
-            plugin.hooks.on_setup(self.config.plugin_opts)
+        if not self.__class__.SETUP:  # pragma: no cover
+            plugin.hooks.on_setup(self.config)
             self.__class__.SETUP = True
 
         self.__class__.PIPELINE_COUNT += 1
@@ -99,26 +103,33 @@ class Pipen:
         """Run the processes one by one
 
         Args:
-            profile: The profile to use
+            profile: The default profile to use for the run
 
         Returns:
             True if the pipeline ends successfully else False
         """
+        self.profile = profile
         self.workdir = Path(self.config.workdir) / slugify(self.name)
+        self.workdir.mkdir(parents=True, exist_ok=True)
 
         succeeded = True
-        self._init(profile)
+        await self._init()
         logger.setLevel(self.config.loglevel.upper())
         log_rich_renderable(pipen_banner(), "magenta", logger.info)
         try:
-            await plugin.hooks.on_init(self)
             self._build_proc_relationships()
-            self._log_pipeline_info(profile)
+            self._log_pipeline_info()
             await plugin.hooks.on_start(self)
 
             for proc in self.procs:
                 self.pbar.update_proc_running()
                 proc_obj = proc(self)  # type: ignore
+                if proc in self.starts and proc.input_data is None:
+                    proc_obj.log(
+                        "warning",
+                        "This is a start process, "
+                        "but no 'input_data' specified.",
+                    )
                 await proc_obj._init()
                 await proc_obj.run()
                 if proc_obj.succeeded:
@@ -130,46 +141,99 @@ class Pipen:
                 proc_obj.gc()
 
             logger.info("")
+        except Exception:
+            raise
+        else:
+            await plugin.hooks.on_complete(self, succeeded)
         finally:
+            self.plugin_context.__exit__()
             if self.pbar:
                 self.pbar.done()
-            await plugin.hooks.on_complete(self, succeeded)
-            self.plugin_context.__exit__()
 
         return succeeded
 
     def run(
         self,
-        *starts: Union[Type[Proc], Sequence[Type[Proc]]],
         profile: str = "default",
     ) -> int:
         """Run the pipeline with the given profile
+        This is just a sync wrapper for the async `async_run` function using
+        `asyncio.run()`
 
         Args:
             profile: The default profile to use for the run
-                Unless the profile is defined in the processes, otherwise
-                this profile will be used
 
         Returns:
             True if the pipeline ends successfully else False
         """
-        self._set_starts(*starts)
         return asyncio.run(self.async_run(profile))
 
-    def _set_starts(self, *procs: Union[Type[Proc], Sequence[Type[Proc]]]):
-        """Set the starts"""
-        for proc in procs:
-            if isinstance(proc, (list, tuple)):
-                self._set_starts(*proc)
-            elif proc not in self.starts:
-                self.starts.append(proc)  # type: ignore
-
-    def _log_pipeline_info(self, profile: str) -> None:
-        """Print the information of the pipeline
+    def set_data(self, *indata: Iterable) -> "Pipen":
+        """Set the input_data for start processes
 
         Args:
-            profile: The profile to print
+            *indata: The input data for the start processes
+                The data will set for the processes in the order determined by
+                `set_starts()`.
+                If a process has input_data set, an error will be raised.
+                To use that input_data, set None here in the corresponding
+                position for the process
+
+        Raises:
+            ProcInputDataError: When trying to set input data to
+                processes with input_data already set
+
+        Returns:
+            `self` to chain the operations
         """
+        for start, data in zip(self.starts, indata):
+            if data is None:
+                continue
+            if start.input_data is not None:
+                raise PipenSetDataError(
+                    f"`input_data` has already set for {start}. "
+                    "If you want to use it, set `None` at the position of "
+                    "this process for `Pipen.set_data()`."
+                )
+            start.input_data = data
+        return self
+
+    def set_starts(
+        self,
+        *procs: Union[Type[Proc], Sequence[Type[Proc]]],
+        clear: bool = True
+    ):
+        """Set the starts
+
+        Args:
+            *procs: The processes to set as starts of the pipeline.
+            clear: Wether to clear previous set starts
+
+        Raises:
+            ProcDependencyError: When processes set as starts repeatedly
+
+        Returns:
+            `self` to chain the operations
+        """
+        if clear:
+            self.starts = []
+
+        for proc in procs:
+            if isinstance(proc, (list, tuple)):
+                self.set_starts(*proc, clear=False)
+            elif proc not in self.starts:
+                self.starts.append(proc)  # type: ignore
+            else:
+                raise ProcDependencyError(
+                    f"{proc} is already a start process."
+                )
+        return self
+
+    # In case people forget the "s"
+    set_start = set_starts
+
+    def _log_pipeline_info(self) -> None:
+        """Print the information of the pipeline"""
         items = (
             [
                 Panel(
@@ -196,7 +260,12 @@ class Pipen:
         for key, value in chain(
             zip(
                 ["# procs", "plugins", "profile", "outdir"],
-                [str(len(self.procs)), enabled_plugins, profile, self.outdir],
+                [
+                    str(len(self.procs)),
+                    enabled_plugins,
+                    self.profile,
+                    self.outdir,
+                ],
             ),
             sorted(self.config.items()),
         ):
@@ -228,7 +297,7 @@ class Pipen:
             logger.info,
         )
 
-    def _init(self, profile: str) -> None:
+    async def _init(self) -> None:
         """Compute the configurations for the pipeline based on the priorities
 
         Configurations (priority from low to high)
@@ -243,9 +312,12 @@ class Pipen:
         # Then load the configurations from config files
         config = Config()
         config._load(*CONFIG_FILES)
-        config._use(profile)
+        config._use(self.profile)
         self.config.update(config)
 
+        # configs from files and CONFIG are loaded
+        # allow plugins to change the default configs
+        await plugin.hooks.on_init(self)
         # Then load the extra configurations passed from __init__(**kwargs)
         # Make sure dict options get inherited
         self.config.template_opts.update(self._kwargs.pop("template_opts", {}))
@@ -258,7 +330,10 @@ class Pipen:
     def _build_proc_relationships(self) -> None:
         """Build the proc relationships for the pipeline"""
         if not self.starts:
-            raise ProcDependencyError("No start processes specified.")
+            raise ProcDependencyError(
+                "No start processes specified. "
+                "Did you forget to call `Pipen.set_starts()`?"
+            )
 
         # build proc relationships
         self.procs = self.starts[:]
@@ -271,7 +346,9 @@ class Pipen:
             # pick up one that can be added to procs
             for proc in sorted(nexts, key=lambda prc: prc.name):
                 if proc in self.procs:
-                    raise ProcDependencyError(f"Cyclic dependency: {proc.name}")
+                    raise ProcDependencyError(
+                        f"Cyclic dependency: {proc.name}"
+                    )
 
                 # Add proc to self.procs if all their requires
                 # are added to self.procs
@@ -288,7 +365,8 @@ class Pipen:
                 if nexts:
                     raise ProcDependencyError(
                         f"No available next processes for {nexts}. "
-                        "Did you forget to start with their required processes?"
+                        "Did you forget to start with their "
+                        "required processes?"
                     )
 
         self.pbar = PipelinePBar(len(self.procs), self.name.upper())
