@@ -98,14 +98,16 @@ class Proc(ABC, metaclass=ProcMeta):
         name: The name of the process. Will use the class name by default.
         desc: The description of the process. Will use the summary from
             the docstring by default.
-        args: The arguments that are job-independent, useful for common options
+        envs: The arguments that are job-independent, useful for common options
             across jobs.
         cache: Should we detect whether the jobs are cached?
         dirsig: When checking the signature for caching, whether should we walk
             through the content of the directory? This is sometimes
             time-consuming if the directory is big.
-        end: When False, force the process not to be the end process, meaning
-            results will not be saved to the output directory of the pipeline.
+        export: When True, the results will be exported to `<pipeline.outdir>`
+            Defaults to None, meaning only end processes will export.
+            You can set it to True/False to enable or disable exporting
+            for processes
         error_strategy: How to deal with the errors
             - retry, ignore, halt
             - halt to halt the whole pipeline, no submitting new jobs
@@ -140,10 +142,10 @@ class Proc(ABC, metaclass=ProcMeta):
 
     name: str = None
     desc: str = None
-    args: Mapping[str, Any] = None
+    envs: Mapping[str, Any] = None
     cache: bool = None
     dirsig: bool = None
-    end: bool = None
+    export: bool = None
     error_strategy: str = None
     num_retries: int = None
     template: Union[str, Type[Template]] = None
@@ -170,9 +172,9 @@ class Proc(ABC, metaclass=ProcMeta):
         proc: Type["Proc"],
         name: str = None,
         desc: str = None,
-        args: Mapping[str, Any] = None,
+        envs: Mapping[str, Any] = None,
         cache: bool = None,
-        end: bool = None,
+        export: bool = None,
         error_strategy: str = None,
         num_retries: int = None,
         forks: int = None,
@@ -189,11 +191,14 @@ class Proc(ABC, metaclass=ProcMeta):
             proc: The Proc subclass
             name: The new name of the process
             desc: The new description of the process
-            args: The arguments of the process, will overwrite parent one
+            envs: The arguments of the process, will overwrite parent one
                 The items that are specified will be inherited
             cache: Whether we should check the cache for the jobs
-            end: When False, force the process not to be an end process
-                if it is a terminal process
+            export: When True, the results will be exported to
+                `<pipeline.outdir>`
+                Defaults to None, meaning only end processes will export.
+                You can set it to True/False to enable or disable exporting
+                for processes
             error_strategy: How to deal with the errors
                 - retry, ignore, halt
                 - halt to halt the whole pipeline, no submitting new jobs
@@ -225,8 +230,8 @@ class Proc(ABC, metaclass=ProcMeta):
         kwargs: Dict[str, Any] = {"name": name}
         if desc is not None:
             kwargs["desc"] = desc
-        if args is not None:
-            kwargs["args"] = update_dict(proc.args, args)
+        if envs is not None:
+            kwargs["envs"] = update_dict(proc.envs, envs)
         if cache is not None:
             kwargs["cache"] = cache
         if forks is not None:
@@ -247,7 +252,7 @@ class Proc(ABC, metaclass=ProcMeta):
         if submission_batch is not None:
             kwargs["submission_batch"] = submission_batch
 
-        kwargs["end"] = end
+        kwargs["export"] = export
         kwargs["input_data"] = input_data
         kwargs["requires"] = requires
         kwargs["nexts"] = None
@@ -272,6 +277,7 @@ class Proc(ABC, metaclass=ProcMeta):
         """
         # instance properties
         self.pipeline = pipeline
+
         self.pbar = None
         self.jobs: List[Any] = []
         self.xqute = None
@@ -279,13 +285,16 @@ class Proc(ABC, metaclass=ProcMeta):
             self.name
         )
 
+        # plugins can modify some default attributes
+        plugin.hooks.on_proc_init(self)
+
         # Compute the properties
         # otherwise, the property can be accessed directly from class vars
         if self.desc is None:
             self.desc: str = desc_from_docstring(self.__class__)
 
-        if self.end is None:
-            self.end = bool(not self.nexts)
+        if self.export is None:
+            self.export = bool(not self.nexts)
 
         # log the basic information
         self._log_info()
@@ -312,13 +321,16 @@ class Proc(ABC, metaclass=ProcMeta):
         # check if it's the same proc using the workdir
         # since the directory name is slugified
         proc_name_file = self.workdir / "proc.name"  # type: ignore
-        if proc_name_file.is_file() and proc_name_file.read_text() != self.name:
+        if (
+            proc_name_file.is_file()
+            and proc_name_file.read_text() != self.name
+        ):
             raise ProcWorkdirConflictException(
                 "Workdir name is conflicting with process "
                 f"{proc_name_file.read_text()!r}, use a differnt pipeline "
                 "workdir or a different process name."
             )
-        self.workdir.mkdir(parents=True, exist_ok=True)
+        self.workdir.mkdir(exist_ok=True)
         proc_name_file.write_text(self.name)
 
         if self.submission_batch is None:
@@ -326,7 +338,9 @@ class Proc(ABC, metaclass=ProcMeta):
 
     async def _init(self) -> None:
         """Init all other properties and jobs"""
-        scheduler_opts = copy_dict(self.pipeline.config.scheduler_opts, 2) or {}
+        scheduler_opts = (
+            copy_dict(self.pipeline.config.scheduler_opts, 2) or {}
+        )
         scheduler_opts.update(self.scheduler_opts or {})
         self.xqute = Xqute(
             self.scheduler,
@@ -378,7 +392,12 @@ class Proc(ABC, metaclass=ProcMeta):
         msg = msg % args
         if not isinstance(level, int):
             level = logging.getLevelName(level.upper())
-        logger.log(level, "[cyan]%s:[/cyan] %s", self.name, msg)  # type: ignore
+        logger.log(
+            level,  # type: ignore
+            "[cyan]%s:[/cyan] %s",
+            self.name,
+            msg,
+        )
 
     async def run(self) -> None:
         """Run the process"""
@@ -397,7 +416,7 @@ class Proc(ABC, metaclass=ProcMeta):
                 job.status = JobStatus.FINISHED
             await self.xqute.put(job)
         if cached_jobs:
-            self.log("info", "Cached jobs: %s", brief_list(cached_jobs))
+            self.log("info", "Cached jobs: [%s]", brief_list(cached_jobs))
         await self.xqute.run_until_complete()
         self.pbar.done()
         await plugin.hooks.on_proc_done(
@@ -443,11 +462,15 @@ class Proc(ABC, metaclass=ProcMeta):
         if is_subclass(requires, Proc):
             requires = [requires]  # type: ignore
 
+        # if req is in cls.__bases__, then cls.nexts will be affected by
+        # req.nexts
+        my_nexts = None if cls.nexts is None else cls.nexts[:]
         for req in requires:  # type: ignore
             if req.nexts is None:
                 req.nexts = [cls]
             else:
                 req.nexts.append(cls)  # type: ignore
+        cls.nexts = my_nexts
 
         return requires  # type: ignore
 
@@ -614,7 +637,7 @@ class Proc(ABC, metaclass=ProcMeta):
                 "║ ║║\n"
                 "╰═┴╯\n"
             )
-            if self.end
+            if self.export
             else box.ROUNDED,
             width=min(CONSOLE_WIDTH, console_width),
         )
